from unipath import Path

from atelier.test import TestCase

import etgen
SRC = Path(etgen.__file__).parent + '/'

class PackagesTests(TestCase):
    def test_packages(self):
        self.run_packages_test(etgen.SETUP_INFO['packages'])


class UtilsTests(TestCase):

    def test_utils(self):
        self.run_simple_doctests(SRC+'utils.py')


    def test_xmlgen_html(self):
        self.run_simple_doctests(SRC+'html.py')

    def test_html2rst(self):
        self.run_simple_doctests(SRC+'html2rst.py')

    def test_xmlgen_sepa(self):
        self.run_simple_doctests(SRC+'sepa/__init__.py')

        



from etgen.html import HtmlNamespace



E = HtmlNamespace(None, set("""
ObjectListItem
SideNavigation
item
NavigationList
NavigationListitem
fixedItem
mvc:View
m:Button

a
abbr
acronym
address
alt
applet
area
b
base
basefont
bdo
big
blockquote
body
br
button
caption
center
cite
code
col
colgroup
dd
del
dfn
dir
div
dl
dt
em
fieldset
font
form
frame
frameset
h1
h2
h3
h4
h5
h6
head
height
hr
html
i
iframe
img
input
ins
isindex
kbd
label
legend
li
link
map
menu
meta
name
noframes
noscript
object
ol
optgroup
option
p
param
pre
q
s
samp
script
select
small
span
strike
strong
style
sub
sup
table
tbody
td
textarea
tfoot
th
thead
title
tr
tt
u
ul
var
class
id
bgcolor
cellspacing
width
align
valign
href
type
rel
target
value
onclick
src
rows
data-toggle
tabindex
placeholder
""".split()))



"""
Based on Amaury's adaption of Eli Golovinsky's post
http://stackoverflow.com/questions/174890/how-to-output-cdata-using-elementtree

Gooli's original snippet didn't work in Python 2.7

Added by LS:
- support for _serialize_xml having different number of args depending on Python version
- register_namespace is not in __all__

"""
import six
import xml.etree.ElementTree as etree

if hasattr(etree, '_serialize_xml'):

    def CDATA(text=None):
        element = etree.Element('![CDATA[')
        element.text = text
        return element

    _original_serialize_xml = etree._serialize_xml

    def _serialize_xml(write, elem, *args, **kwargs):
        if elem.tag == '![CDATA[':
            write("\n<%s%s]]>\n" % (
                elem.tag, elem.text))
            return
        return _original_serialize_xml(write, elem, *args, **kwargs)
    etree._serialize_xml = etree._serialize['xml'] = _serialize_xml

    register_namespace = etree.register_namespace

else:
    def register_namespace(*args, **kw):
        pass

from xml.etree.ElementTree import *

if six.PY3:
    _tostring = tostring
    # # change default value for encoding to 'unicode'
    # def tostring(element, encoding="us-ascii", *args, **kwargs):
    #     return _tostring(element, encoding, *args, **kwargs)
    def tostring(*args, **kwargs):
        return _tostring(*args, **kwargs).decode()



# -*- coding: UTF-8 -*-
# Copyright 2013-2018 Rumma & Ko Ltd
# License: BSD (see file COPYING for details)


u"""A set of generator tags for building SEPA documents.

Requires Python 2.7 because it needs the `default_namespace` for
`ElementTree.write`.

Usage:

>>> from etgen.sepa import E
>>> x = E.pain_001_001_02(
...   E.GrpHdr(
...     E.MsgId("SAL63023CP20130621022043"),
...     E.CreDtTm("2013-06-21T02:20:43") ,
...     E.NbOfTxs("8") ,
...     E.Grpg("MIXD") ,
...     E.InitgPty(E.Nm(u"ÖSHZ Nispert")) )
...   )
>>> print (E.tostring_pretty(x))
<pain.001.001.02 xmlns="urn:iso:std:iso:20022:tech:xsd:pain.001.001.02">
<GrpHdr>
<MsgId>SAL63023CP20130621022043</MsgId>
<CreDtTm>2013-06-21T02:20:43</CreDtTm>
<NbOfTxs>8</NbOfTxs>
<Grpg>MIXD</Grpg>
<InitgPty>
<Nm>&#214;SHZ Nispert</Nm>
</InitgPty>
</GrpHdr>
</pain.001.001.02>

"""

from __future__ import unicode_literals

import logging
logger = logging.getLogger(__name__)

import os
import sys
import cgi
import datetime

from xml.etree import ElementTree as ET
from etgen import etree
from etgen.utils import Namespace, RAW

# names have been extracted using:
# python xsdspy.py XSD/pain.001.001.02.xsd
# python xsdspy.py XSD/pain.001.001.05.xsd
# name Grpg seems to be used only in pain.001.001.02 but not in pain.001.001.05

E = Namespace("urn:iso:std:iso:20022:tech:xsd:pain.001.001.02", """
ExternalPersonIdentification1Code Rcrd GenericPersonIdentification1 PrvcOfBirth BatchBookingIndicator RltdDt Envlp UnitCcy OrgId GroupHeader48 CtryOfRes Adr Document FrDt GenericAccountIdentification1 CdtrRefInf TaxRecordPeriod1Code IntrmyAgt2Acct DlvrTo AnyBIC Ustrd CtrctId FwdgAgt PmtInfId CtrlSum AddtlRmtInf PaymentTypeInformation19 OrganisationIdentificationSchemeName1Choice ExternalPurpose1Code ExternalLocalInstrument1Code Othr InstrPrty Nb TaxTp Invcr DuePyblAmt ExternalFinancialInstitutionIdentification1Code AdrTp TaxInformation3 MobNb DbtrAcct PstlAdr PmtInf AdmstnZn ReferredDocumentInformation3 IntrmyAgt3Acct DocumentType5Code ExchangeRate1 PhoneNumber Issr Ctgy ExternalClearingSystemIdentification1Code Dt CityOfBirth FrmsCd FaxNb Prd InstrForDbtrAgt ActiveOrHistoricCurrencyAndAmount_SimpleType SvcLvl DateAndPlaceOfBirth ContactDetails2 Tax EquivalentAmount2 DlvryMtd ExternalCashAccountType1Code Mtd CertId ChequeType2Code DocumentType3Code MemoFld PstCd CtgyDtls TaxAuthorisation1 ChqInstr Max2048Text ToDt PercentageRate CdtNoteAmt IntrmyAgt1Acct DiscountAmountType1Choice EndToEndId ChqNb ActiveOrHistoricCurrencyCode InstdAmt RmtId RmtdAmt ExternalDiscountAmountType1Code ExternalAccountIdentification1Code Cd Ref RemittanceLocationMethod2Code CtctDtls AnyBICIdentifier ISODateTime CreditorReferenceType2 CountryCode ChqFr CreDtTm TaxId TtlTaxblBaseAmt PrtLctn TtlTaxAmt AmountType3Choice RmtInf Dtls Priority2Code Yr DatePeriodDetails DecimalNumber PmtTpInf TwnNm DbtrSts Ccy ChrgsAcctAgt OrganisationIdentification8 ReferredDocumentType2 Rsn CdtTrfTxInf PrvtId Dept ChequeDelivery1Code RfrdDocAmt SchmeNm TaxAmount1 TaxParty1 RefNb PlcAndNm Max140Text BldgNb BirthDt StrtNm BICFIIdentifier CashAccount24 CreditTransferTransaction6 Cdtr Authorisation1Code PaymentInstruction9 Max128Text TaxAmountAndType1 DbtrAgt SupplementaryDataEnvelope1 Strd AdjstmntAmtAndRsn CustomerCreditTransferInitiationV05 RgltryRptg CdOrPrtry Purp XchgRateInf Nm RemittanceInformation7 Titl Max35Text GenericFinancialIdentification1 NmPrfx RegulatoryReportingType1Code ActiveOrHistoricCurrencyAndAmount PersonIdentification5 Authorisation1Choice MsgId CdtrAgtAcct ChargeBearerType1Code LocalInstrument2Choice RegulatoryAuthority2 CdtrAcct TaxParty2 ExternalOrganisationIdentification1Code Max34Text BICFI InstrInf Max15NumericText FinancialInstitutionIdentification8 Max16Text RmtLctnPstlAdr PartyIdentification43 BranchAndFinancialInstitutionIdentification5 Authrty IBAN2007Identifier ChequeDeliveryMethod1Choice TaxRecord1 ExchangeRateType1Code BranchData2 SubDept ClearingSystemMemberIdentification2 StructuredRegulatoryReporting3 TaxAmountType1Choice CdtrAgt EqvtAmt UltmtDbtr Dbtr InstrForCdtrAgt Authstn ChqTp IBAN ChrgBr PoolgAdjstmntDt TaxblBaseAmt CdtDbtInd AdrLine LclInstrm DtAndPlcOfBirth Prtry AccountSchemeName1Choice BrnchId Purpose2Choice CategoryPurpose1Choice XchgRate RmtLctnMtd Number InstrId Amt TaxRecordDetails1 CtryOfBirth PersonIdentificationSchemeName1Choice PaymentMethod3Code DscntApldAmt Inf SupplementaryData1 PostalAddress6 ChrgsAcct PmtId CtrySubDvsn RegulatoryReporting3 CreditorReferenceType1Choice CreditorReferenceInformation2 EmailAdr RemittanceAmount2 RltdRmtInf RfrdDocInf PaymentIdentification1 Tp Sgntr ClearingSystemIdentification2Choice CashAccountType2Choice ISODate FinancialIdentificationSchemeName1Choice ReqdExctnDt UltmtCdtr DbtCdtRptgInd NameAndAddress10 SplmtryData Invcee DbtrAgtAcct CtgyPurp GrpHdr TaxAmt PmtMtd ReferredDocumentType1Choice Ctry ExternalServiceLevel1Code RmtLctnElctrncAdr BtchBookg Max10Text NamePrefix1Code CreditDebitCode BaseOneRate InstructionForCreditorAgent1 ExternalCategoryPurpose1Code Instruction3Code ChqMtrtyDt ClrSysId SeqNb PhneNb TaxPeriod1 CcyOfTrf RgnlClrZone ExternalTaxAmountType1Code AddressType2Code FinInstnId MmbId AccountIdentification4Choice NbOfTxs TtlAmt RegnId Party11Choice Max70Text IntrmyAgt3 IntrmyAgt2 IntrmyAgt1 Cheque7 Max4Text Max350Text InitgPty DiscountAmountAndType1 GenericOrganisationIdentification1 RemittanceLocation2 StructuredRemittanceInformation9 Rate ClrSysMmbId ServiceLevel8Choice AddtlInf DocumentAdjustment1 RateTp CstmrCdtTrfInitn FrToDt Id
FinancialInstitutionIdentification5Choice SclSctyNb ReferredDocumentAmount1Choice TaxRefNb PrvcOfBirth BatchBookingIndicator DocumentType2Code OrgId RfrdDocNb CtryOfRes Adr Document AuthrtyCtry IntrmyAgt2Acct DlvrTo Ustrd CtrctId FwdgAgt PaymentInstructionInformation1 PmtInfId TtlTaxAmt CdtrTaxTp CreditTransferTransactionInformation1 CdtTrfTxInf DtAndPlcOfBirth InstrPrty TaxTp GroupHeader1 TaxInformation2 PaymentCategoryPurpose1Code IdntyCardNb DbtrTaxId CHIPSUniversalIdentifier PstlAdr CurrencyCode CashAccountType2 IntrmyAgt3Acct Issr Max3Text PrvtId RfrdDocRltdDt PsptNb CityOfBirth FrmsCd InstrForDbtrAgt LclInstrm LocalInstrument1Choice SvcLvl GenericIdentification4 TaxTpInf TaxDt IBEI Tax DbtrAcct DlvryMtd PersonIdentification3 Purpose1Choice CdtrRef MsgId DocumentType3Code NmAndAdr AmountType2Choice MemoFld PstCd OthrId DunsIdentifier ChqInstr InstrInf PercentageRate CdtNoteAmt IntrmyAgt1Acct ChrgsAcctAgt RfrdDocTp EndToEndId ChqNb InstdAmt RmtId RmtdAmt CashAccount7 Cd PartyIdentification8 ISODateTime CountryCode ChqFr CreDtTm Grpg TtlTaxblBaseAmt PrtLctn AlnRegnNb ServiceLevel2Choice RmtInf Priority2Code DecimalNumber PmtTpInf PrtryId Ccy BkPtyId TaxIdNb ReferredDocumentType1 AccountIdentification3Choice Ctry CdtrAcct OrganisationIdentification2 ChequeDelivery1Code RfrdDocAmt NameAndAddress3 BEIIdentifier Inf Max140Text EANGLNIdentifier BldgNb BirthDt StrtNm DuePyblAmt Cdtr CmbndId Max128Text Dbtr DbtrAgt ChrgsAcct RgltryRptg RemittanceInformation1 StructuredRemittanceInformation6 Purp XchgRateInf Nm Max35Text UPIC RegulatoryReporting2 RegulatoryReportingType1Code ReferredDocumentInformation1 DateAndPlaceOfBirth BEI GenericIdentification3 AdrTp ClearingSystemMemberIdentification3Choice CdtrAgtAcct ChargeBearerType1Code CreditorReferenceInformation1 AddtlRmtInf BranchData EquivalentAmount FinancialInstitutionIdentification3 Max34Text IBEIIdentifier NameAndAddress7 Party2Choice Max15NumericText Max16Text RmtLctnPstlAdr DrvrsLicNb BranchAndFinancialInstitutionIdentification3 Authrty CtgyDesc ExternalClearingSystemMemberCode ChequeDeliveryMethod1Choice ExchangeRateType1Code CdtrTaxId Strd ExternalPurposeCode CdtrAgt Cheque5 EqvtAmt UltmtDbtr TaxblBaseAmt BICIdentifier InstrForCdtrAgt RemittanceLocationMethod1Code Authstn ChqTp IBAN ChrgBr PoolgAdjstmntDt DUNS ExternalLocalInstrumentCode ClrChanl AdrLine pain.001.001.02 CurrencyAndAmount BrnchId ServiceLevel1Code ChequeType2Code XchgRate BBAN InstrId Amt CtryOfBirth PaymentMethod3Code DscntApldAmt SimpleIdentificationInformation2 CurrencyAndAmount_SimpleType PostalAddress1 Invcr CdtrRefTp PmtId CtrySubDvsn PrtryAcct RltdRmtInf RfrdDocInf PaymentIdentification1 Tp IdTp Prtry Max256Text CertId ISODate ReqdExctnDt UltmtCdtr DbtCdtRptgInd StructuredRegulatoryReporting2 Instruction3Code AuthrtyNm Invcee DbtrAgtAcct CtgyPurp GrpHdr TaxAmt PmtMtd BIC EANGLN IBANIdentifier BtchBookg PmtInf CtrlSum BaseOneRate InstructionForCreditorAgent1 BBANIdentifier CdtrRefInf RmtLctnElctrncAdr ChqMtrtyDt ClearingChannel2Code CashAccountType4Code USCHU UPICIdentifier ExchangeRateInformation1 CcyOfTrf RgnlClrZone RgltryDtls TaxType AddressType2Code FinInstnId NbOfTxs Max70Text IntrmyAgt3 IntrmyAgt2 IntrmyAgt1 CreditorReferenceType1 PaymentTypeInformation1 RmtLctnMtd InitgPty RemittanceLocation1 Rate ClrSysMmbId RegulatoryAuthority MplyrIdNb CstmrNb RateTp TwnNm TaxDetails Id Grouping1Code
""".split())


def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from atelier.test import make_docs_suite

def load_tests(loader, standard_tests, pattern):
    return make_docs_suite(
        "docs", exclude="docs/specs/printing.rst",
        addenv=dict(LINO_LOGLEVEL="INFO"))




# -*- coding: UTF-8 -*-
# doctest etgen/html.py
# Adapted copy from lxml\src\lxml\html\builder.py
# --------------------------------------------------------------------
# The ElementTree toolkit is
# Copyright (c) 1999-2004 by Fredrik Lundh
# Modifications in this file are
# Copyright (c) 2012-2019 Rumma & Ko Ltd
# --------------------------------------------------------------------

"""Defines an ElementTree Builder for generating HTML documents.

.. data:: E

    The global ElementTree Builder object.

Usage:

>>> # from etgen.html import E
>>> import lxml.usedoctest
>>> html = E.html(
...            E.head( E.title("Hello World") ),
...            E.body(
...              E.h1("Hello World !"),
...              CLASS("main")
...            )
...        )

>>> print (tostring_pretty(html))
<html>
  <head>
    <title>Hello World</title>
  </head>
  <body class="main">
    <h1>Hello World !</h1>
  </body>
</html>


>>> kw = dict(title=u'Ein süßes Beispiel')
>>> kw.update(href="foo/bar.html")
>>> btn = E.button(type='button', **CLASS('x-btn-text x-tbar-upload'))
>>> html = E.a(btn, **kw)
>>> print (tostring_pretty(html))
<a href="foo/bar.html" title="Ein süßes Beispiel">
  <button class="x-btn-text x-tbar-upload" type="button"/>
</a>

You can also do the opposite, i.e. parse HTML:

>>> from lxml import etree
>>> E_raw = etree.fromstring
>>> html = E_raw('''<a href="foo/bar.html"
... title="Ein s&#252;&#223;es Beispiel">
... <button class="x-btn-text x-tbar-upload" type="button"/>
... </a>''')
>>> print(tostring_pretty(html))
<a href="foo/bar.html" title="Ein süßes Beispiel">
  <button class="x-btn-text x-tbar-upload" type="button"/>
</a>


>>> print(tostring(E_raw(
...     '<ul type="disc"><li>First</li><li>Second</li></ul>')))
<ul type="disc"><li>First</li><li>Second</li></ul>

>>> html = E.div(E.p("First"), E.p("Second"))
>>> print(tostring_pretty(html))
<div>
  <p>First</p>
  <p>Second</p>
</div>
>>> html.attrib['class'] = 'htmlText'
>>> print(tostring_pretty(html))
<div class="htmlText">
  <p>First</p>
  <p>Second</p>
</div>

Avoid self-closing tags
=======================

lxml generates self-closing tags for elements without children:

>>> print(tostring(E.div()))
<div/>

Some environments refuse empty ``div>`` elements and interpret a ``<div/>`` as
``<div>`` (don't ask me why).  You can avoid the self-closing tag by setting
the ``text`` attribute to an empty string:

>>> html = E.div()
>>> html.text = ""
>>> print(tostring(html))
<div></div>

Note that you must set ``text`` explicitly. Simply specifying it when
instantiating the element is not enough:

>>> print(tostring(E.div("")))
<div/>

>>> print(tostring(E.div(" ")))
<div/>

The real solution would be to use the "html" method when writing the tree to
html:

>>> print(tostring(E.div(), method="html"))
<div></div>

TODO: This approach has been active as default value (see disabled line in code
of :func:`tostring`) and I don't remember why we disabled it.  I suggest to
re-enable it and test thoroughly whether this causes regressions (and if yes,
why it causes them).




"""

from __future__ import unicode_literals
from builtins import str
from builtins import object

import types
from xml.etree import ElementTree as ET
# from lxml.etree import HTML
from lxml import etree
from lxml.etree import iselement
from etgen.utils import join_elems, forcetext
# from etgen.utils import Namespace
from etgen.html2rst import html2rst, UnsupportedHtmlTag
# from htmlentitydefs import name2codepoint

# ENTITIES = {}
# ENTITIES.update((x, unichr(i)) for x, i in name2codepoint.iteritems())


# def CreateParser():
#     """Every string that is being parsed must get its own parser instance.
#     This is because "Due to limitations in the Expat library used by
#     pyexpat, the xmlparser instance returned can only be used to parse
#     a single XML document. Call ParserCreate for each document to
#     provide unique parser instances. (`docs.python.org
#     <https://docs.python.org/2/library/pyexpat.html>`_)

#     """
#     p = ET.XMLParser()
#     # PARSER.entity.update(htmlentitydefs.entitydefs)
#     p.entity = ENTITIES
#     assert 'otilde' in p.entity
#     assert 'eacute' in p.entity
#     assert 'nbsp' in p.entity
#     return p

# class RAW_HTML_STRING(unicode):
#     pass


def CLASS(*args): # class is a reserved word in Python
    return {"class": ' '.join(args)}

# class HtmlNamespace(Namespace):
#     """The HTML namespace.
#     This is instantiated as ``E``.
#     """

def tostring(v, *args, **kw):
    # if isinstance(v, types.GeneratorType):
    if isinstance(v, (types.GeneratorType, list, tuple)):
        return "".join([tostring(x, *args, **kw) for x in v])
    if etree.iselement(v):
        # kw.setdefault('method', 'html')
        kw.setdefault('encoding', 'unicode')
        return etree.tostring(v, *args, **kw)
    return str(v)

def tostring_pretty(*args, **kw):
    kw.setdefault('pretty_print', True)
    return tostring(*args, **kw).strip()  # remove blank line at end
    # return prettify(s)


def to_rst(v, stripped=True):
    if isinstance(v, types.GeneratorType):
        return "".join([to_rst(x, stripped) for x in v])
    if iselement(v):
        try:
            return html2rst(v, stripped)
        except UnsupportedHtmlTag as e:
            raise Exception("{} while converting {} to reSTructuredText".format(e, tostring(v)))
    return str(v)

#     # def raw(self, raw_html):
#     #     return RAW_HTML_STRING(raw_html)

#     def raw(self, raw_html):
#         """
#         Parses the given string into an HTML Element.

#         It the string contains a a single top-level element, then this
#         element is returned. Otherwise return the wrapping ``body``
#         element.
#         """
#         # print 20151008, raw_html

#         # the lxml parser wraps `<html><body>...</body></html>` around
#         # the snippet, but we don't want it.
#         try:
#             root = HTML(raw_html)[0]
#             # root = E.html(raw_html)[0]
#         except Exception as e:
#             return E.p("Invalid HTML ({}) in {}".format(e, raw_html))
#         if len(root) == 1:
#             return root[0]
#         return root
#         # try:
#         #     return self.fromstring(raw_html, parser=CreateParser())
#         # except ET.ParseError as e:
#         #     raise Exception("ParseError {0} in {1}".format(e, raw_html))

from lxml.builder import E

# E = HtmlNamespace(None, set("""
# a
# abbr
# acronym
# address
# alt
# applet
# area
# b
# base
# basefont
# bdo
# big
# blockquote
# body
# br
# button
# caption
# center
# cite
# code
# col
# colgroup
# dd
# del
# dfn
# dir
# div
# dl
# dt
# em
# fieldset
# font
# form
# frame
# frameset
# h1
# h2
# h3
# h4
# h5
# h6
# head
# height
# hr
# html
# i
# iframe
# img
# input
# ins
# isindex
# kbd
# label
# legend
# li
# link
# map
# menu
# meta
# name
# noframes
# noscript
# object
# ol
# optgroup
# option
# p
# param
# pre
# q
# s
# samp
# script
# select
# small
# span
# strike
# strong
# style
# sub
# sup
# table
# tbody
# td
# textarea
# tfoot
# th
# thead
# title
# tr
# tt
# u
# ul
# var

# class
# id
# bgcolor
# cellspacing
# width
# align
# valign
# href
# type
# rel
# target
# value
# onclick
# src
# rows
# data-toggle
# tabindex
# placeholder
# """.split()))


def table_header_row(*headers, **kw):
    return E.tr(*[E.th(h, **kw) for h in headers])


def table_body_row(*cells, **kw):
    return E.tr(*[E.td(h, **kw) for h in cells])


class Table(object):
    """A pythonic representation of a ``<table>`` with ``<head>``,
    ``<foot>`` and ``<body>``.

    """
    def __init__(self):
        self.clear()

    def clear(self):
        self.head = []
        self.foot = []
        self.body = []
        self.attrib = dict()

    def add_header_row(self, *args, **kw):
        e = table_header_row(*args, **kw)
        self.head.append(e)
        return e

    def add_footer_row(self, *args, **kw):
        e = table_body_row(*args, **kw)
        self.foot.append(e)
        return e

    def add_body_row(self, *args, **kw):
        e = table_body_row(*args, **kw)
        self.body.append(e)
        return e

    def as_element(self):
        children = []
        if self.head:
            children.append(E.thead(*self.head))
        if self.foot:
            children.append(E.tfoot(*self.foot))
        if self.body:
            children.append(E.tbody(*self.body))
        return E.table(*children, **self.attrib)


class Document(object):
    """A pythonic representation of a ``<body>`` with a ``<title>`` and
    some ``<head>`` tags for stylesheets.

    """

    def __init__(self, title, stylesheets=None):
        self.title = title
        self.body = []
        self.stylesheets = stylesheets or []

    def add_stylesheet(self, url):
        self.stylesheets.append(url)

    def add_table(self):
        t = Table()
        self.body.append(t)
        return t

    def write(self, *args, **kw):
        ET.ElementTree(self.as_element()).write(*args, **kw)

    def as_element(self):
        body = []
        for e in self.body:
            if isinstance(e, Table):
                body.append(e.as_element())
            else:
                body.append(e)
        headers = []
        for css in self.stylesheets:
            headers.append(E.link(rel="stylesheet", type="text/css", href=css))
        headers.append(E.title(self.title))

        return E.html(
            E.head(*headers),
            E.body(*body)
        )


def lines2p(lines, min_height=0, **attrs):
    """
    Convert the given list of text lines `lines` into a paragraph
    (``<p>``) with one ``<br>`` between each line. If optional
    `min_height` is given, add empty lines if necessary.

    Examples:

    >>> print(tostring(lines2p(['first', 'second'])))
    <p>first<br/>second</p>

    >>> print(tostring(lines2p(['first', 'second'], min_height=5)))
    <p>first<br/>second<br/><br/><br/></p>

    If `min_height` is specified, and `lines` contains more items,
    then we don't truncate:

    >>> print(tostring(lines2p(['a', 'b', 'c', 'd', 'e'], min_height=4)))
    <p>a<br/>b<br/>c<br/>d<br/>e</p>

    This also works:

    >>> print(tostring(lines2p([], min_height=5)))
    <p><br/><br/><br/><br/></p>
    """
    while len(lines) < min_height:
        lines.append('')
    lines = join_elems(lines, E.br)
    return E.p(*lines, **attrs)


def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from os.path import join, dirname
from atelier.test import TestCase
from etgen.sepa.validate import validate_pain001

class SepaTests(TestCase):
    def test_sepa(self):
        fn = join(dirname(__file__), 'finan.PaymentOrder-63.xml')
        validate_pain001(fn)
